// Demonstration of sending data over a TCP connection, which consists of:
//  1. Looking up a DNS hostname to find its corresponding IP address
//  2. Allocating a TCP socket
//  3. Opening a connection to the server using its address and a provided port
//     (This also allocates a local ephemeral port and binds the socket to it.)
//  -> Performing any desired data transmission <-
//  4. Deallocating the linked list of addresses populated by the DNS request.
//
// Note that in order to keep this demo simple, a few corners were cut:
//   - It is assumed that the first of the IP addresses returned by the DNS
//     query is the desired one.  Real implementations should traverse the
//     linked list returned by getaddrinfo() in search of the working address.
//   - The size of packets to send out is left up to the system's TCP protocol
//     stack.  In practice, this means that large amounts of input will be
//     batched before transmission in order to minimize network traffic;
//     consequently, reasonably-sized messages won't be displayed on the server
//     until the stream is terminated.  The upshot of this is that you either
//     need to pipe or redirect input into the application
//
//       (e.g. $ echo input | ./server hostname port
//             -or-
//             $ ./server hostname port <file)
//
//     or press return followed by Ctrl-D at the end of your keyboard input.
//
// Author: Sol Boucher <sboucher@cmu.edu>

#include <sys/socket.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char **argv) {
	if(argc != 3) {
		printf("USAGE: %s <hostname> <port>\n", argv[0]);
		return 1;
	}
	char *hostname = argv[1];
	char *port = argv[2];

	// 1. getaddrinfo(name, port, params, list)
	struct addrinfo params = {
		.ai_family = AF_INET,
		.ai_socktype = SOCK_STREAM,
	};
	struct addrinfo *addr = NULL;
	int stat;
	if((stat = getaddrinfo(hostname, port, &params, &addr)) || !addr) {
		fprintf(stderr, "%s: %s\n", "getaddrinfo()", gai_strerror(stat));
		return 2;
	}

	// 2. socket(family, socktype, transport)
	int fd;
	if((fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		perror("socket()");
		return 3;
	}

	// 3. connect(fd, addr, size)
	if(connect(fd, addr->ai_addr, addr->ai_addrlen)) {
		perror("connect()");
		return 4;
	}

	FILE *conn = fdopen(fd, "w");
	if(!conn) {
		perror("fdopen()");
		return 5;
	}

	int chr;
	while((chr = getchar()) != EOF)
		putc(chr, conn);

	fclose(conn);

	// 4. freeaddrinfo(list)
	freeaddrinfo(addr);

	return 0;
}
